#ifndef MODULES_H
#define MODULES_H

#include <qstring.h>
#include <qobject.h>
#include <qstringlist.h>
#include <qdialog.h>
#include <qlistview.h>
#include <qtranslator.h>
#include <qlabel.h>
#include <qhbox.h>

/**
	Zastpuje klas QLibrary na specyficzne potrzeby Kadu
**/
class Library : public QObject
{
	private:
		QString FileName;
		void* Handle;
		
	public:
		Library(const QString& file_name);
		~Library();
		bool load();
		void* resolve(const QString& symbol_name);
		QString error();
};

/**
 informacje o module
**/
struct ModuleInfo
{
	/**
	 jakie inne moduy s wymagane przez ten modu
	**/
	QStringList depends;
	/**
	 z jakimi moduami ten modu konfliktuje
	**/
	QStringList conflicts;
	/**
	 jak cech dostarcza ten modu
	**/
	QStringList provides;
	/**
	 opis
	**/
	QString description;
	/**
	 autor moduu
	**/
	QString author;
};

class ModulesDialog : public QHBox
{
	Q_OBJECT
	
	private:
		QListView* lv_modules;
		QLabel* l_moduleinfo;
		
	private slots:
		void loadItem();
		void unloadItem();
		void moduleAction(QListViewItem *);
		void itemsChanging();
		void getInfo();
		void refreshList();
		void keyPressEvent(QKeyEvent *);
	
	public:
		ModulesDialog();
		~ModulesDialog();
};

class ModulesManager : public QObject
{
	Q_OBJECT

	private:
		typedef int InitModuleFunc();
		typedef void CloseModuleFunc();
		/**
			Informacje o statycznym module.
		**/
		struct StaticModule
		{
			InitModuleFunc* init;
			CloseModuleFunc* close;
		};
		/**
			Lista statycznych moduw wypeniania
			przez kod generowany przez configure.
		**/
		QMap<QString,StaticModule> StaticModules;
		/**
			Informacje o aktywnym module
			statycznym bd zewntrznym.
			Dla moduu statycznego lib==NULL.
		**/
		struct Module
		{
			Library* lib;
			CloseModuleFunc* close;
			QTranslator* translator;
			ModuleInfo info;
			int usage_counter;
		};
		/**
			Lista aktywnych moduw
			statycznych bd zewntrznych.
		**/
		QMap<QString,Module> Modules;	
		/**
		**/
		ModulesDialog* Dialog;
		/**
			aduje plik z tumaczeniem. Zwraca NULL jeli wystpi
			bd.
		**/
		QTranslator* loadModuleTranslation(const QString& module_name);
		/**
			Sprawdza czy dostpne s moduy z listy
			zalenoci danego moduu. W razie czego
			stara si je zaadowa jeli s dostpne.
			@param module_info informacje o module
		**/
		bool satisfyModuleDependencies(const ModuleInfo& module_info);
		/**
			Zwiksza liczniki uycia moduw uywanych
			przez dany modu.
			@param module_info informacje o module
		**/
		void incDependenciesUsageCount(const ModuleInfo& module_info);
		/**
			Rejestruje modu statyczny. Funcja wywoywana
			dla wszystkich moduw statycznych przez kod
			wygenerowany przez configure.
		**/
		void registerStaticModule(const QString& module_name,
			InitModuleFunc* init,CloseModuleFunc* close);
		/**
			Rejestruje moduy statyczne. Kod funkcji jest
			generowany przez configure.
		**/
		void registerStaticModules();

	private slots:
		void dialogDestroyed();
	
	public:	
		static void initModule();
		static void closeModule();
		ModulesManager();
		~ModulesManager();
		/**
			Zwraca list moduw wkompilowanych
			statycznie w plik wykonywalny kadu
		**/
		QStringList staticModules() const;
		/**
			Zwraca list moduw zainstalowanych jako
			dzielone biblioteki (shared libraries)
		**/
		QStringList installedModules() const;
		/**
			Zwraca list moduw zaadowanych do pamici
			jako dzielone biblioteki (shared libraries)
		**/
		QStringList loadedModules() const;
		/**
			Zwraca list moduw zainstalowanych jako
			dzielone biblioteki (shared libraries)
			i nie zaadowanych aktualnie do pamici
		**/		
		QStringList unloadedModules() const;
		/**
			Zwraca list aktywnych moduw.
			Uwzgldniane s zarwno aktywne moduy
			statyczne jak i zewntrzne.
		**/
		QStringList activeModules() const;
		/**
			Pobiera do info informacje o module module_name jesli
			sie to udalo zwraca true w przeciwnym wypadku false		 
		**/		
		bool moduleInfo(const QString& module_name, ModuleInfo& info) const;
		/**
			Sprawdza czy podany modu jest wkompilowany statycznie.
			@param module_name nazwa moduu
		**/
		bool moduleIsStatic(const QString& module_name) const;
		/**
			Sprawdza czy podany modu jest zainstalowany
			w katalogu z moduami zewntrznymi.
			@param module_name nazwa moduu
		**/
		bool moduleIsInstalled(const QString& module_name) const;
		/**
			Sprawdza czy podany modu zewntrzny jest zaadowany
			@param module_name nazwa moduu
		**/		
		bool moduleIsLoaded(const QString& module_name) const;
		/**
			Sprawdza czy podany modu jest aktywny.
			Dziaa dla moduw statycznych i zaadowanych
			zewntrznych.
			@param module_name nazwa moduu
		**/
		bool moduleIsActive(const QString& module_name) const;
		/**
			Sprawdza czy podany modu konfliktuje
			z jakim innym zaadowanym moduem
			@param module_name nazwa moduu
			@param module_info informacje o module
		**/
		bool conflictsWithLoaded(const QString &module_name, const ModuleInfo& module_info) const;
		
	public slots:
		void showDialog();
		/**
		 	Zwiksza licznik uycia moduu o 1
			@param module_name nazwa moduu
		**/
		void moduleIncUsageCount(const QString& module_name);
		/**
		 	Zmniejsza licznik uycia moduu o 1
			@param module_name nazwa moduu
		**/
		void moduleDecUsageCount(const QString& module_name);
		/**
			Zapisuje do pliku konfiguracyjnego list zaadowanych
			moduw.
		**/
		void saveLoadedModules();
		/**
			Aktywuje modu statyczny jeli jest dostpny
			lub aduje do pamici i aktywuje modu zewntrzny.
			@param module_name nazwa moduu
		**/
		bool activateModule(const QString& module_name);
		/**
			Deaktywuje modu statyczny lub
			deaktywuje i usuwa z pamici modu zewntrzny.
			@param module_name nazwa moduu
		**/		
		bool deactivateModule(const QString& module_name, bool force=false);
};

extern ModulesManager* modules_manager;

#endif
